{% load cache %}{% cache None lazy_focuspoint_picture img img.get_focal_point classes %}{% spaceless %}<picture class="lazy">
    <data-src type="image/webp" srcset="{{ img2560_webp.url }} {{ img2560_webp.width }}w{% if img1200_webp %}, {{ img1200_webp.url }} {{ img1200_webp.width }}w{% endif %}{% if img768_webp %}, {{ img768_webp.url }} {{ img768_webp.width}}w{% endif %}"></data-src>
    <data-src srcset="{{ img2560.url }} {{ img2560.width }}w{% if img1200 %}, {{ img1200.url }} {{ img1200.width }}w{% endif %}{% if img768 %}, {{ img768.url }} {{ img768.width}}w{% endif %}"></data-src>
    <data-img src="{{ img2560.url }}" alt="{{ img2560.alt }}" width="{{ img2560.width }}" height="{{ img2560.height }}" class="{{ classes }}" ></data-img>
</picture>
{% endspaceless %}{% endcache %}