# Generated by Django 3.2.20 on 2023-10-13 08:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('main', '0002_alter_cronlock_pid'),
    ]

    operations = [
        migrations.CreateModel(
            name='Maintenance',
            fields=[
                ('key', models.CharField(max_length=255, primary_key=True, serialize=False, unique=True)),
                ('result', models.TextField()),
            ],
        ),
    ]
