# Generated by Django 3.2.23 on 2024-01-22 13:03

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('main', '0003_maintenance'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='maintenance',
            options={'ordering': ('pk',)},
        ),
    ]
