{% load sorl_thumbnail cache %}
{% cache None owl_lazy_picture image geometry format alt upscale classes options %}{% spaceless %}
{% thumbnail image geometry format=format upscale=upscale options=options as thumb_default %}
<picture>
    {% thumbnail image geometry upscale=upscale format="WEBP" options=options as thumb_webp %}
    {% include "ocyan/thumbnail/owl_lazy_source.html" with type="image/webp" thumb=thumb_webp %}
    {% endthumbnail %}

    {% include "ocyan/thumbnail/owl_lazy_source.html" with thumb=thumb_default %}
    {% include "ocyan/thumbnail/owl_lazy_img.html" with thumb=thumb_default %}
</picture>
{% endthumbnail %}
{% endspaceless %}{% endcache %}