# Generated by Django 3.2.19 on 2023-08-15 08:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('main', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cronlock',
            name='pid',
            field=models.PositiveBigIntegerField(blank=True, null=True),
        ),
    ]
