# Generated by Django 3.2.19 on 2023-07-10 12:49

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CronLock',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('jobname', models.CharField(max_length=255, unique=True)),
                ('started', models.DateTimeField(blank=True, null=True)),
                ('pid', models.PositiveIntegerField(blank=True, null=True)),
            ],
        ),
    ]
