from django.http import HttpResponse
from django.urls import include, path


def admin_index(request):
    return HttpResponse()


admin_patterns = ([path("", admin_index, name="index")], "admin")
urlpatterns = [path("manage/", include(admin_patterns, namespace="admin"))]
