from pathlib import Path

from setuptools import find_packages, setup

PACKAGE_CLASSIFIERS = [
    "Framework :: Ocyan",
]

install_requires: list = [
    "ocyan.core>=1.2.7",
    "django>=5,<6",
    "django-oscar",
    "django-oscar-api",
    "django-compressor",
    "django-libsass",
    "beautifulsoup4",
    "lxml",
    "polib",
    "sorl-thumbnail",
]

extras_require: dict = {
    "test": [
        "wheel",
        "empty_testproject",
        "pylint-django",
        "isort",
        "ruff",
        "pillow",
        "ocyan.plugin.testing",
        "vdt.versionplugin.wheel",
        "coverage",
        "ocyan.plugin.oscar_paid_product_options",
        "ocyan.plugin.seo",
        "oxyan.themes",
    ]
}

setup(
    name="ocyan.main",
    version="2.0.1",
    url="https://git.mandelblog.com/mandel-plugins/ocyan.main",
    author="Mandel",
    author_email="info@mandelblog.com",
    description="Integrates and configures ocyan plugins in projects.",
    long_description=Path("README.rst").read_text(encoding="utf-8"),
    long_description_content_type="text/x-rst",
    include_package_data=False,
    packages=find_packages(include=["ocyan", "ocyan.*"]),
    package_data={"ocyan.main": ["templates/**/*", "locale/**/*"]},
    python_requires=">=3.12,<3.13",
    install_requires=install_requires,
    classifiers=PACKAGE_CLASSIFIERS,
    extras_require=extras_require,
)
